/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  DocPanel.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package snmpmanager.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import snmpmanager.*;
import snmpmanager.images.*;
import toolkit.*;
import toolkit.gui.*;

/** Panel permettant l'affichage de la documentation */
public class DocPanel extends DefaultPanel {
    
    /** Champs de texte contenant l'url du document affich */
    private JTextField m_txtUrl = new JTextField(50);
    /** Bouton "Prcdent" */
    private RolloverButton m_cmdPrev = new RolloverButton(Utilities.getIcon(Icons.ICON_BACK, SNMPManager.class),"Prev");
    /** Bouton "Suivant" */
    private RolloverButton m_cmdNext = new RolloverButton(Utilities.getIcon(Icons.ICON_NEXT, SNMPManager.class),"Next");
    /** Browser HTML permettant l'affichage de la doc */
    private HTMLViewer m_viewer = new HTMLViewer();
    /** Arbre reprsentant le sommaire de la doc */
    private ContentsTree m_contentsTree = new ContentsTree();
    
    /* URLs des fichiers de la documentation */
    static final String INSTALL = "/../docs/html/install.htm";
    static final String ABOUT = "/../docs/html/about.htm";
    static final String GNU = "/../docs/html/gnu.htm";
    static final String MIB = "/../docs/html/mib.xml";
    
    /** Constructeur
     * @param logPanel nouveau panel de log
     * @param frame fentre principale de l'application
     */
    public DocPanel(MainFrame frame, LogPanel logPanel) {
        super(frame, logPanel);
        
        JScrollPane scrollContents = new JScrollPane(m_contentsTree);
        
        JPanel pnlContents = new JPanel(new BorderLayout());
        pnlContents.add(scrollContents,BorderLayout.CENTER);
        pnlContents.setPreferredSize(new Dimension(200,200));
        JScrollPane scrollViewer = new JScrollPane(m_viewer);
        pnlContents.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Contents"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        JToolBar toolBar = new JToolBar();
        toolBar.add(m_cmdPrev);
        toolBar.add(m_cmdNext);
        toolBar.addSeparator();
        toolBar.add(new JLabel("  Location :  "));
        toolBar.add(m_txtUrl);
        toolBar.setFloatable(false);
        
        this.setLayout(new BorderLayout(5,5));
        this.add(scrollViewer, BorderLayout.CENTER);
        this.add(pnlContents, BorderLayout.WEST);
        this.add(toolBar, BorderLayout.NORTH);
        
        m_cmdPrev.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                m_viewer.prevPage();
            }
        });
        
        m_cmdNext.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                m_viewer.nextPage();
            }
        });
        
        try {
            m_viewer.setPage(new File(SNMPManager.getAppliHome() + ABOUT).toURL());
        }
        catch(Exception e) { m_viewer.setText("Documentation not found:\n"+e.toString()); }
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        m_contentsTree = null;
        m_viewer = null;
        m_txtUrl = null;
        m_cmdPrev = null;
        m_cmdNext = null;
        super.finalize();
    }
    
    /** Zone de texte permettant l'affichage des fichiers HTML et la gestion des
     *  pages prcdentes et suivantes
     */
    private final class HTMLViewer extends JEditorPane {
        
        private Stack m_prevStack = new Stack();
        private Stack m_nextStack = new Stack();
        
        public HTMLViewer() {
            this.setEditable(false);
            this.setFont(new Font("Dialog", Font.PLAIN, 12));
        }
        
        public void setPage(URL url) {
            if (url!=null)
                try {
                    super.setPage(url);
                    m_prevStack.push(url);
                    m_txtUrl.setText(url.toString());
                }
                catch(IOException e) { this.setText("Documentation not found:\n"+e.toString()); }
        }
        
        public void prevPage() {
            if (m_prevStack.size() > 1) {
                m_nextStack.push((URL)m_prevStack.pop());
                setPage((URL)m_prevStack.peek());
                m_prevStack.pop();
            }
        }
        
        public void nextPage() {
            if (m_nextStack.size() > 0) {
                setPage((URL)m_nextStack.pop());
            }
        }
    }
    
    /** Arbre reprsentant le sommaire de la doc */
    private final class ContentsTree extends JTree implements TreeSelectionListener {
        
        private final class ContentsURL {
            public String m_name;
            public URL m_url = null;
            
            public ContentsURL(String n,String path) {   
                m_name = n;
                if (path != null)
                    try {
                        m_url = new File(SNMPManager.getAppliHome() + path).toURL();
                    }
                    catch(Exception e) { path = null; }
            }
            public String toString() { return m_name; }
        }
        
        public ContentsTree() {
            this.setFont(new Font("Dialog",Font.PLAIN,11));
            DefaultMutableTreeNode ndeRoot = new DefaultMutableTreeNode("");
            DefaultMutableTreeNode ndeInstall = new DefaultMutableTreeNode(new ContentsURL("Install",INSTALL));
            DefaultMutableTreeNode ndeAbout = new DefaultMutableTreeNode(new ContentsURL("About",ABOUT));
            DefaultMutableTreeNode ndeGnu = new DefaultMutableTreeNode(new ContentsURL("Licence GNU",GNU));
            DefaultMutableTreeNode ndeMib = new DefaultMutableTreeNode(new ContentsURL("MIB",MIB));
            
            ndeRoot.add(ndeAbout);
            ndeRoot.add(ndeInstall);
            ndeRoot.add(ndeGnu);
            ndeRoot.add(ndeMib);
            
            this.setModel(new DefaultTreeModel(ndeRoot));
            this.setRootVisible(false);
            this.expandRow(4);
            this.expandRow(5);
            this.setShowsRootHandles(true);
            this.putClientProperty("JTree.lineStyle","Angled");
            this.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
            this.setPreferredSize(new Dimension(180,180));
            this.setMinimumSize(new Dimension(180,180));
            this.addTreeSelectionListener(this);
        }
        
        public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
            TreePath path = this.getSelectionPath();
            if (path == null) return;
            DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)path.getLastPathComponent();
            ContentsURL c = (ContentsURL)selectedNode.getUserObject();
            m_viewer.setPage(c.m_url);
        }
    }
}

