/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  Utilities.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package toolkit;

import java.util.*;
import java.text.SimpleDateFormat;
import java.io.*;
import java.awt.*;
import java.net.*;
import javax.swing.*;

/** Class regroupant diffrentes fonctions utiles
 */
public class Utilities {
    
    /** Ouvrir une fentre <I>dialog</I> permettant  l'utilisateur d'entrer une donn
     * @param parent composant appelant cette fentre
     * @param message message  afficher dans la fentre
     * @param titre titre de la fentre
     * @return retourne la valeur entre par l'utilisateur
     */
    public static String showInputQuestion(Component parent, String message, String titre) {
        return JOptionPane.showInputDialog(parent,message,titre,JOptionPane.QUESTION_MESSAGE);
    }
    
    /** Affiche un message d'erreur
     * @param message message  afficher
     */
    public static void showError(String message) {
        JOptionPane.showMessageDialog(null, message, "Erreur", JOptionPane.ERROR_MESSAGE);
    }
    
    /** Affiche une fentre permettant de demander  l'utilisateur si il veut enregistrer un fichier
     * @param parent composant appelant cette fentre
     * @param name nom du fichier
     * @return entier correspondant aux boutons <I>yes no</I> et <I>cancel</I> (voir <CODE>JOptionPane</CODE>)
     */
    public static int showWantToSaveFile(Component parent, String name) {
        return JOptionPane.showConfirmDialog(parent,"Voulez-vous enregistrer le fichier "+name+" ?","Attention",JOptionPane.YES_NO_CANCEL_OPTION);
    }
    
    /** Affiche un message dans une fentre <I>dialog</I>
     * @param title titre
     * @param message message  afficher
     */
    public static void showMessage(String title, String message) {
        JOptionPane.showMessageDialog(null, message, title, JOptionPane.INFORMATION_MESSAGE);
    }
    
    /** Permet d'mettre un son "beep"
     */
    public static void beep() {
        Toolkit.getDefaultToolkit().beep();
    }
    
    /** Cration d'un objet <CODE>Image</CODE>  partir d'un fichier
     * @param picture chemin d'accs relatif de l'image
     * @param source class  partir de laquelle le chemin d'accs est recherch
     * @return l'image
     */
    public static Image getImage(String picture, Class source) {
        return Toolkit.getDefaultToolkit().getImage(source.getResource(picture));
    }
    
    /** Cration d'un objet <CODE>URL</CODE>  partir d'un fichier
     * @param path chemin d'accs relatif du fichier
     * @param source class  partir de laquelle le chemin d'accs est recherch
     * @return url du fichier
     */
    public static URL getURL(String path, Class source) {
        return source.getResource(path);
    }
    
    /** Cration d'un objet <CODE>ImageIcon</CODE>  partir d'un fichier
     * @param picture chemin d'accs relatif de l'image
     * @param source class  partir de laquelle le chemin d'accs est recherch
     * @return icne
     */
    public static ImageIcon getIcon(String picture, Class source) {
        return new ImageIcon(source.getResource(picture));
    }
    
    /** Renvoie les coordonnes pour que le second composant soit positionn au milieu du premier
     * @return coordonnes
     * @param comp1 premier composant
     * @param comp2 second composant
     */
    public static Point getCenterComponentChild(Component comp1, Component comp2) {
        return new Point(comp1.getX()+comp1.getWidth()/2-comp2.getWidth()/2,comp1.getY()+comp1.getHeight()/2-comp2.getHeight()/2);
    }
    
    /** Renvoie les coordonnes pour que le composant soit au centre de l'cran
     * @return coordonnes
     * @param comp composant  centrer
     */
    public static Point getCenterComponent(Component comp) {
        Dimension reso = comp.getToolkit().getScreenSize();
        return new Point(reso.width/2-comp.getWidth()/2,reso.height/2-comp.getHeight()/2);
    }
    
    /** Renvoie l'extension d'un fichier pass en paramtre
     * @param f fichier
     * @return extension du fichier
     */
    public final static String getFileExtension(File f) {
        String path = f.getName();
        int i = path.lastIndexOf('.');
        if (i<0)
            return "";
        return path.substring(i);
    }
    
    /** Renvoie l'extension d'un fichier dont le chemin est pass en paramtre
     * @param fp chemin du fichier
     * @return extension du fichier
     */
    public final static String getFileExtension(String fp) {
        int i = fp.lastIndexOf('.');
        if (i<0)
            return "";
        return fp.substring(i);
    }
    
    /** Permet de changer le <I>look & feel</I> de l'interface
     * @param c composant dont il faut changer le look
     * @param plaf nom du <I>look & feel</I> ("metal", "motif", "windows", "macintosh")
     */
    public static void changePlaf(Component c, String plaf) {
        plaf = plaf.toLowerCase();
        JOptionPane DialogBox = new JOptionPane("Info",JOptionPane.ERROR_MESSAGE);
        try {
            
            if (plaf.equals("metal"))
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            if (plaf.equals("cde/motif"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            if (plaf.equals("windows"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            SwingUtilities.updateComponentTreeUI(c);
            
        }
        catch (UnsupportedLookAndFeelException e) {
            String s = "Look & Feel non disponible sur votre systme.\n";
            if (plaf.equals("windows"))
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf.equals("mac"))
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(c,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (IllegalAccessException e) {
            String s = "Look & Feel non accessible sur votre systme.\n";
            if (plaf.equals("windows"))
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf.equals("mac"))
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(c,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (ClassNotFoundException e) {
            String s = "Look & Feel non trouv.\n";
            if (plaf.equals("windows"))
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf.equals("mac"))
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(c,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (InstantiationException e) {
            System.out.println
            ("Ce Look & Feel ne peut tre instanci.");
        }
        catch (Exception e) {
            System.out.println("Erreur d'excution.");
        }
        DialogBox = null;
    }
    
    /**
     * Gets a string containing current date and time.
     *
     * @return a string containing the date and time.
     */
    public static String getTimestamp() {
        return (new SimpleDateFormat("HH:mm:ss:")).format(new Date());
    }
    
    /** Teste si une adresse IP de type XXX.XXX.XXX.XXX est valide
     * @param address adresse IP
     */
    public static boolean isValidIPAddress(String address) {
        boolean res = true;
        StringTokenizer st = new StringTokenizer(address,".");
        int c = 0;
        try {
            while(st.hasMoreTokens()) {
                c++;
                String token = st.nextToken();
                int i = Integer.parseInt(token);
                if (i<0 || i>255) return false;
            }
        }
        catch (NumberFormatException nfe) {
            return false;
        }
        if (c==4)
        return true;
        else return false;
    }
}